@@ -83,7 +83,7 @@ static int on_expose(TickitWindow *win, TickitEventFlags flags, void *_info, voi
 
 int main(int argc, char *argv[])
 {
-  Tickit *t = tickit_new_stdio();
+  Tickit *t = tickit_new_stdtty();
 
   TickitWindow *root = tickit_get_rootwin(t);
   if(!root) {
