@@ -25,8 +25,8 @@ extern "C" {
 #endif
 
 #define TICKIT_VERSION_MAJOR 0
-#define TICKIT_VERSION_MINOR 3
-#define TICKIT_VERSION_PATCH 5
+#define TICKIT_VERSION_MINOR 4
+#define TICKIT_VERSION_PATCH 1
 
 int tickit_version_major(void);
 int tickit_version_minor(void);
@@ -48,6 +48,9 @@ typedef struct Tickit Tickit;
 /* Opaque struct pointers required but not part of official API */
 typedef struct TickitTermDriver TickitTermDriver;
 
+/* Forward declaration to pointer defined by <tickit-evloop.h> */
+typedef struct TickitEventHooks TickitEventHooks;
+
 typedef struct {
   int top;
   int left;
@@ -63,6 +66,7 @@ typedef enum {
   TICKIT_BIND_FIRST   = 1<<0,
   TICKIT_BIND_UNBIND  = 1<<1,
   TICKIT_BIND_DESTROY = 1<<2,
+  TICKIT_BIND_ONESHOT = 1<<3,
 } TickitBindFlags;
 
 typedef enum {
@@ -77,6 +81,19 @@ typedef enum {
   TICKIT_CURSORSHAPE_LEFT_BAR,
 } TickitCursorShape;
 
+typedef enum {
+  TICKIT_IO_IN    = 1<<0,
+  TICKIT_IO_OUT   = 1<<1,
+  TICKIT_IO_HUP   = 1<<2,
+  TICKIT_IO_ERR   = 1<<3,
+  TICKIT_IO_INVAL = 1<<4,
+} TickitIOCondition;
+
+typedef struct {
+  int fd;
+  TickitIOCondition cond;
+} TickitIOWatchInfo;
+
 typedef enum {
   TICKIT_LINECAP_START = 0x01,
   TICKIT_LINECAP_END   = 0x02,
@@ -109,6 +126,13 @@ typedef enum {
   TICKIT_N_PEN_ATTRS
 } TickitPenAttr;
 
+/* additional attribute types recognised by tickit_pen_new_attrs */
+enum {
+  /* We're unlikely to ever have 256 attributes, so adding 0x100 should be safe */
+  TICKIT_PEN_FG_DESC = 0x100 + TICKIT_PEN_FG,
+  TICKIT_PEN_BG_DESC = 0x100 + TICKIT_PEN_BG,
+};
+
 typedef enum {
   TICKIT_PEN_UNDER_NONE,
   TICKIT_PEN_UNDER_SINGLE,
@@ -385,10 +409,30 @@ TickitTerm *tickit_term_new(void);
 TickitTerm *tickit_term_new_for_termtype(const char *termtype);
 void tickit_term_destroy(TickitTerm *tt);
 
-// Internal use only for now but consider making it public API
+typedef void TickitTermOutputFunc(TickitTerm *tt, const char *bytes, size_t len, void *user);
+
 struct TickitTermBuilder {
   const char *termtype;
 
+  enum {
+    TICKIT_NO_OPEN,
+    TICKIT_OPEN_FDS,    /* use input_fd, output_fd */
+    TICKIT_OPEN_STDIO,  /* input=0, output=1 */
+    TICKIT_OPEN_STDTTY, /* input = output = first of 0/1/2 for which isatty() is true */
+    /* TODO: Consider
+     *   TICKIT_OPEN_DEVTTY to open /dev/tty
+     */
+  } open;
+  int input_fd, output_fd; /* only valid if open==TICKIT_OPEN_FDS */
+
+  TickitTermOutputFunc *output_func;
+  void                 *output_func_user;
+
+  size_t output_buffersize;
+
+  /* Fields below here are undocumented and for vaguely internal or
+   * special-case purposes
+   */
   TickitTermDriver *driver;
 
   const struct TickitTerminfoHook {
@@ -398,6 +442,8 @@ struct TickitTermBuilder {
 };
 TickitTerm *tickit_term_build(const struct TickitTermBuilder *builder);
 
+void tickit_term_teardown(TickitTerm *tt);
+
 TickitTerm *tickit_term_ref(TickitTerm *tt);
 void        tickit_term_unref(TickitTerm *tt);
 
@@ -405,8 +451,6 @@ TickitTerm *tickit_term_open_stdio(void);
 
 const char *tickit_term_get_termtype(TickitTerm *tt);
 
-typedef void TickitTermOutputFunc(TickitTerm *tt, const char *bytes, size_t len, void *user);
-
 void tickit_term_set_output_fd(TickitTerm *tt, int fd);
 int  tickit_term_get_output_fd(const TickitTerm *tt);
 void tickit_term_set_output_func(TickitTerm *tt, TickitTermOutputFunc *fn, void *user);
@@ -708,10 +752,23 @@ typedef int TickitCallbackFn(Tickit *t, TickitEventFlags flags, void *info, void
 
 Tickit *tickit_new_for_term(TickitTerm *tt);
 Tickit *tickit_new_stdio(void);
+Tickit *tickit_new_stdtty(void);
 
 Tickit *tickit_ref(Tickit *t);
 void    tickit_unref(Tickit *t);
 
+struct TickitBuilder {
+  TickitTerm *tt;
+  struct TickitTermBuilder term_builder; /* used if tt == NULL */
+
+  /* Fields below here are undocumented and for vaguely internal or
+   * special-case purposes
+   */
+  const TickitEventHooks *evhooks;
+  void *evinitdata;
+};
+Tickit *tickit_build(const struct TickitBuilder *builder);
+
 TickitTerm *tickit_get_term(Tickit *t);
 
 TickitWindow *tickit_get_rootwin(Tickit *t);
@@ -729,7 +786,9 @@ TickitCtl tickit_lookup_ctl(const char *name);
 
 TickitType tickit_ctltype(TickitCtl ctl);
 
-/* TODO: Consider OUT and HUP conditions too */
+void *tickit_watch_io(Tickit *t, int fd, TickitIOCondition cond, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
+
+/* Discouraged synonyn for tickit_watch_io cond=TICKIT_IO_IN|TICKIT_IO_HUP */
 void *tickit_watch_io_read(Tickit *t, int fd, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
 
 void *tickit_watch_timer_after_msec(Tickit *t, int msec, TickitBindFlags flags, TickitCallbackFn *fn, void *user);
@@ -742,11 +801,6 @@ void *tickit_watch_later(Tickit *t, TickitBindFlags flags, TickitCallbackFn *fn,
 
 void tickit_watch_cancel(Tickit *t, void *watch);
 
-/* Entirely undocumented right now */
-void tickit_hook_terminfo(Tickit *t,
-    const char *(*getstr)(const char *name, const char *value, void *data),
-    void         *data);
-
 /* Debug support */
 
 void tickit_debug_init(void);
